==> ampersand-input-view-1.2.2.tgz: package/ampersand-input-view.js <==
var View = require('ampersand-view');


module.exports = View.extend({
    template: [
        '<label>',
            '<span role="label"></span>',
            '<input>',
            '<div role="message-container" class="message message-below message-error">',
                '<p role="message-text"></p>',
            '</div>',
        '</label>'
    ].join(''),
    bindings: {
        'name': {
            type: 'attribute',
            selector: 'input, textarea',
            name: 'name'
        },
        'label': [
            {
                role: 'label'
            },
            {
                type: 'toggle',
                role: 'label'
            }
        ],
        'message': {
            type: 'text',
            role: 'message-text'
        },
        'showMessage': {
            type: 'toggle',
            role: 'message-container'
        },
        'placeholder': {
            type: 'attribute',
            selector: 'input, textarea',
            name: 'placeholder'
        },
        'validityClass': {
            type: 'class',
            selector: 'input, textarea'
        }
    },
    initialize: function (spec) {
        this.tests = this.tests || spec.tests || [];
        this.on('change:type', this.handleTypeChange, this);
        this.handleBlur = this.handleBlur.bind(this);
        this.handleInputChanged = this.handleInputChanged.bind(this);
        this.startingValue = this.value;
        this.on('change:valid change:value', this.reportToParent, this);
    },
    render: function () {
        this.renderWithTemplate();
        this.input = this.get('input') || this.get('textarea');
        // switches out input for textarea if that's what we want
        this.handleTypeChange();
        this.initInputBindings();
        this.setValue(this.value);
    },
    props: {
        value: 'any',
        startingValue: 'any',
        name: 'string',
        type: ['string', true, 'text'],
        placeholder: ['string', true, ''],
        label: ['string', true, ''],
        required: ['boolean', true, true],
        directlyEdited: ['boolean', true, false],
        shouldValidate: ['boolean', true, false],
        message: ['string', true, ''],
        requiredMessage: ['string', true, 'This fields is required.'],
        validClass: ['string', true, 'input-valid'],
        invalidClass: ['string', true, 'input-invalid']
    },
    derived: {
        valid: {
            deps: ['value'],
            fn: function () {
                return !this.runTests();
            }
        },
        showMessage: {
            deps: ['message', 'shouldValidate'],
            fn: function () {
                return this.shouldValidate && this.message;
            }
        },
        changed: {
            deps: ['value', 'startingValue'],
            fn: function () {
                return this.value !== this.startingValue;
            }
        },
        validityClass: {
            deps: ['valid', 'validClass', 'invalidClass', 'shouldValidate', 'changed'],
            fn: function () {
                if (!this.shouldValidate || !this.changed) {
                    return '';
                } else {
                    return this.valid ? this.validClass : this.invalidClass;
                }
            }
        }
    },
    setValue: function (value) {
        this.value = value;
        if (!this.value) {
            this.input.value = '';
        } else {
            this.input.value = this.value.toString();
        }
        if (!this.getErrorMessage(this.value)) {
            this.shouldValidate = true;
        }
    },
    getErrorMessage: function () {
        var message = '';
        if (this.required && !this.value) {
            return this.requiredMessage;
        } else {
            (this.tests || []).some(function (test) {
                message = test.call(this, this.value) || '';
                return message;
            }, this);
            return message;
        }
    },
    handleTypeChange: function () {
        if (this.type === 'textarea' && this.input.tagName.toLowerCase() !== 'textarea') {
            var parent = this.input.parentNode;
            var textarea = document.createElement('textarea');
            parent.replaceChild(textarea, this.input);
            this.input = textarea;
            this._applyBindingsForKey('');
        } else {
            this.input.type = this.type;
        }
    },
    handleInputChanged: function () {
        if (document.activeElement === this.input) {
            this.directlyEdited = true;
        }
        this.value = this.clean(this.input.value);
    },
    clean: function (val) {
        return (this.type === 'number') ? Number(val) : val.trim();
    },
    handleBlur: function () {
        if (this.value && this.changed) {
            this.shouldValidate = true;
        }
        this.runTests();
    },
    beforeSubmit: function () {
        // at the point where we've tried
        // to submit, we want to validate
        // everything from now on.
        this.shouldValidate = true;
        this.runTests();
    },
    runTests: function () {
        var message = this.getErrorMessage();
        if (!message && this.value && this.changed) {
            // if it's ever been valid,
            // we want to validate from now
            // on.
            this.shouldValidate = true;
        }
        this.message = message;
        return message;
    },
    initInputBindings: function () {
        this.input.addEventListener('blur', this.handleBlur, false);
        this.input.addEventListener('input', this.handleInputChanged, false);
    },
    remove: function () {
        this.input.removeEventListener('input', this.handleInputChanged, false);
        this.input.removeEventListener('blur', this.handleBlur, false);
        View.prototype.remove.apply(this, arguments);
    },
    reportToParent: function () {
        if (this.parent) this.parent.update(this);
    }
});

==> ampersand-input-view-2.0.1.tgz: package/ampersand-input-view.js <==
var View = require('ampersand-view');


module.exports = View.extend({
    template: [
        '<label>',
            '<span data-hook="label"></span>',
            '<input class="form-input">',
            '<div data-hook="message-container" class="message message-below message-error">',
                '<p data-hook="message-text"></p>',
            '</div>',
        '</label>'
    ].join(''),
    bindings: {
        'name': {
            type: 'attribute',
            selector: 'input, textarea',
            name: 'name'
        },
        'label': [
            {
                hook: 'label'
            },
            {
                type: 'toggle',
                hook: 'label'
            }
        ],
        'message': {
            type: 'text',
            hook: 'message-text'
        },
        'showMessage': {
            type: 'toggle',
            hook: 'message-container'
        },
        'placeholder': {
            type: 'attribute',
            selector: 'input, textarea',
            name: 'placeholder'
        },
        'validityClass': {
            type: 'class',
            selector: 'input, textarea'
        }
    },
    initialize: function (spec) {
        this.tests = this.tests || spec.tests || [];
        this.on('change:type', this.handleTypeChange, this);
        this.handleBlur = this.handleBlur.bind(this);
        this.handleInputChanged = this.handleInputChanged.bind(this);
        this.startingValue = this.value;
        this.on('change:valid change:value', this.reportToParent, this);
    },
    render: function () {
        this.renderWithTemplate();
        this.input = this.query('input') || this.query('textarea');
        // switches out input for textarea if that's what we want
        this.handleTypeChange();
        this.initInputBindings();
        this.setValue(this.value);
    },
    props: {
        value: 'any',
        startingValue: 'any',
        name: 'string',
        type: ['string', true, 'text'],
        placeholder: ['string', true, ''],
        label: ['string', true, ''],
        required: ['boolean', true, true],
        directlyEdited: ['boolean', true, false],
        shouldValidate: ['boolean', true, false],
        message: ['string', true, ''],
        requiredMessage: ['string', true, 'This field is required.'],
        validClass: ['string', true, 'input-valid'],
        invalidClass: ['string', true, 'input-invalid']
    },
    derived: {
        valid: {
            deps: ['value'],
            fn: function () {
                return !this.runTests();
            }
        },
        showMessage: {
            deps: ['message', 'shouldValidate'],
            fn: function () {
                return this.shouldValidate && this.message;
            }
        },
        changed: {
            deps: ['value', 'startingValue'],
            fn: function () {
                return this.value !== this.startingValue;
            }
        },
        validityClass: {
            deps: ['valid', 'validClass', 'invalidClass', 'shouldValidate', 'changed'],
            fn: function () {
                if (!this.shouldValidate || !this.changed) {
                    return '';
                } else {
                    return this.valid ? this.validClass : this.invalidClass;
                }
            }
        }
    },
    setValue: function (value) {
        this.value = value;
        if (!this.value) {
            this.input.value = '';
        } else {
            this.input.value = this.value.toString();
        }
        if (!this.getErrorMessage(this.value)) {
            this.shouldValidate = true;
        }
    },
    getErrorMessage: function () {
        var message = '';
        if (this.required && !this.value) {
            return this.requiredMessage;
        } else {
            (this.tests || []).some(function (test) {
                message = test.call(this, this.value) || '';
                return message;
            }, this);
            return message;
        }
    },
    handleTypeChange: function () {
        if (this.type === 'textarea' && this.input.tagName.toLowerCase() !== 'textarea') {
            var parent = this.input.parentNode;
            var textarea = document.createElement('textarea');
            parent.replaceChild(textarea, this.input);
            this.input = textarea;
            this._applyBindingsForKey('');
        } else {
            this.input.type = this.type;
        }
    },
    handleInputChanged: function () {
        if (document.activeElement === this.input) {
            this.directlyEdited = true;
        }
        this.value = this.clean(this.input.value);
    },
    clean: function (val) {
        return (this.type === 'number') ? Number(val) : val.trim();
    },
    handleBlur: function () {
        if (this.value && this.changed) {
            this.shouldValidate = true;
        }
        this.runTests();
    },
    beforeSubmit: function () {
        // at the point where we've tried
        // to submit, we want to validate
        // everything from now on.
        this.shouldValidate = true;
        this.runTests();
    },
    runTests: function () {
        var message = this.getErrorMessage();
        if (!message && this.value && this.changed) {
            // if it's ever been valid,
            // we want to validate from now
            // on.
            this.shouldValidate = true;
        }
        this.message = message;
        return message;
    },
    initInputBindings: function () {
        this.input.addEventListener('blur', this.handleBlur, false);
        this.input.addEventListener('input', this.handleInputChanged, false);
    },
    remove: function () {
        this.input.removeEventListener('input', this.handleInputChanged, false);
        this.input.removeEventListener('blur', this.handleBlur, false);
        View.prototype.remove.apply(this, arguments);
    },
    reportToParent: function () {
        if (this.parent) this.parent.update(this);
    }
});
